/**
 * 
 */
package com.rjconsultores.ventaboletos.relatorios.impl;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;

import org.apache.log4j.Logger;

import com.rjconsultores.ventaboletos.entidad.PuntoVenta;
import com.rjconsultores.ventaboletos.relatorios.negocio.CalculoImposto;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.DataSource;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.ExceptionConfiguracao;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.Relatorio;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioComissaoSinteticoBean;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.utilerias.MoneyHelper;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;


public class RelatorioComissaoSintetico extends Relatorio  {
	

private  Logger log;
private List<RelatorioComissaoSinteticoBean> lsDadosRelatorio;
private LinkedHashMap<String, LinkedHashMap<String, Object>> mapCacheConfigComissao;
private Set<PuntoVenta> pontoVendaConfiguracao; 


	
	public RelatorioComissaoSintetico(Map<String, Object> parametros, Connection conexao)  throws Exception {
		super(parametros, conexao);
		log = Logger.getLogger(RelatorioComissaoSintetico.class);
		CalculoImposto.limpaCache();
		this.mapCacheConfigComissao = new LinkedHashMap<String, LinkedHashMap<String, Object>>();
		
		this.setCustomDataSource(new DataSource(this) {
			
			@Override
			public void initDados() throws Exception {

				Connection conexao = this.relatorio.getConexao();

				Map<String, Object> parametros = this.relatorio.getParametros();
				String empresaID = parametros.get("EMPRESA_ID") != null ? parametros.get("EMPRESA_ID").toString() : "";
				String puntoVentaID = parametros.get("NUMPUNTOVENTA").equals("-1") ? "" : parametros.get("NUMPUNTOVENTA").toString();
				String sqlTotaisVendas = getSqlDadosTotaisdeVenda(empresaID,puntoVentaID);

				NamedParameterStatement stmt = new NamedParameterStatement(conexao, sqlTotaisVendas);

				
				stmt.setTimestamp("DATA_INICIAL", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
				stmt.setTimestamp("DATA_FINAL", new Timestamp(DateUtil.fimFecha((Date) parametros.get("DATA_FINAL")).getTime()));

				if(empresaID != null && !empresaID.equals("")){
					stmt.setInt("EMPRESA_ID", Integer.parseInt(empresaID));
				}
				pontoVendaConfiguracao = new HashSet<PuntoVenta>();
				ResultSet rset = stmt.executeQuery();
				lsDadosRelatorio = new ArrayList<RelatorioComissaoSinteticoBean>();
				
				while (rset.next()) {
					// Busca as configurações de comissão
					LinkedHashMap<String, Object> configComissao = getConfigComissao(rset.getInt("puntoventa_id"), rset.getInt("empresa_id"));
					Integer agenciaID = rset.getInt("puntoventa_id");
					Integer retEmpresaID = rset.getInt("empresa_id");
					Integer estadoId =  rset.getInt("estado_id");
					Date fecCorrida = rset.getDate("feccorrida");
					BigDecimal totalPassgens =  rset.getBigDecimal("total_passagem") == null ?  BigDecimal.ZERO :  rset.getBigDecimal("total_passagem");
					BigDecimal totalImpPost = rset.getBigDecimal("total_imp_posterior") == null ?  BigDecimal.ZERO : rset.getBigDecimal("total_imp_posterior");
					BigDecimal totalVendaImpPost = rset.getBigDecimal("total_venda_imp_posterior") == null ? BigDecimal.ZERO : rset.getBigDecimal("total_venda_imp_posterior");
					BigDecimal totalSeguro = rset.getBigDecimal("total_seguro") == null ?  BigDecimal.ZERO : rset.getBigDecimal("total_seguro");
					BigDecimal totalVenda = rset.getBigDecimal("total_venda") == null ?  BigDecimal.ZERO : rset.getBigDecimal("total_venda");
					RelatorioComissaoSinteticoBean existingBean = findExistingBean(lsDadosRelatorio, agenciaID, retEmpresaID);
					if (existingBean != null) {
	                    // Registro já existe, adicionar os valores aos existentes
	                    existingBean.setTotalTarifa(existingBean.getTotalTarifa().add(totalPassgens));
	                    existingBean.setTotalImpPosterior(existingBean.getTotalImpPosterior().add(totalImpPost));
	                    existingBean.setTotalVendaImpPosterior(existingBean.getTotalVendaImpPosterior().add(totalVendaImpPost));
	                    existingBean.setTotalSeguro(existingBean.getTotalSeguro().add(totalSeguro));
	                    existingBean.setTotalPagado(existingBean.getTotalPagado().add(totalVenda));
						//aplica comissão dos totais
						if (configComissao != null) {
							comissaoCalc(configComissao, retEmpresaID, estadoId, fecCorrida, totalPassgens, totalImpPost, totalVendaImpPost, totalSeguro, 
									null, existingBean, false);
						}
					} else {
					// Registro não existe, criar um novo objeto e adicioná-lo à lista	
					RelatorioComissaoSinteticoBean comissaoBean = new RelatorioComissaoSinteticoBean();
					
					comissaoBean.setPontoVendaID(agenciaID);
					comissaoBean.setEmpresaID(retEmpresaID);
					comissaoBean.setPontoVenda(rset.getString("nombpuntoventa"));
					comissaoBean.setTotalTarifa(totalPassgens);
					comissaoBean.setTotalImpPosterior(totalImpPost);
					comissaoBean.setTotalVendaImpPosterior(totalVendaImpPost);
					comissaoBean.setTotalOutros(BigDecimal.ZERO);
					comissaoBean.setTotalExcBagagem(BigDecimal.ZERO);
					comissaoBean.setTotalSeguro(totalSeguro);
					comissaoBean.setTotalPagado(totalVenda);
					comissaoBean.setComissaoTarifa(BigDecimal.ZERO);
					comissaoBean.setComissaoVendaImpPosterior(BigDecimal.ZERO);
					comissaoBean.setComissaoImpPosterior(BigDecimal.ZERO);
					comissaoBean.setComissaoExcBagagem(BigDecimal.ZERO);
					comissaoBean.setComissaoOutros(BigDecimal.ZERO);
					
					//aplica comissão dos totais
					if (configComissao != null) {
						comissaoCalc(configComissao, retEmpresaID, estadoId, fecCorrida, totalPassgens, totalImpPost, totalVendaImpPost, totalSeguro,  
								null, comissaoBean, false);
					}
					
					lsDadosRelatorio.add(comissaoBean);
					}
				}
				
				if (lsDadosRelatorio.size() > 0) {
					String sqlTotaisEeventos = getSqlDadosTotaisEvento(empresaID,puntoVentaID);
					NamedParameterStatement stmtEventos = new NamedParameterStatement(conexao, sqlTotaisEeventos);
					stmtEventos.setTimestamp("DATA_INICIAL", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
					stmtEventos.setTimestamp("DATA_FINAL", new Timestamp(DateUtil.inicioFecha((Date) parametros.get("DATA_INICIAL")).getTime()));
					if(empresaID != null && !empresaID.equals("")){
						stmtEventos.setInt("EMPRESA_ID", Integer.parseInt(empresaID));
					}
					ResultSet rsetEventos = stmtEventos.executeQuery();
					
					while(rsetEventos.next()) {
						LinkedHashMap<String, Object> configComissao = getConfigComissao(rset.getInt("puntoventa_id"), rset.getInt("empresa_id"));
						Integer agenciaID = rsetEventos.getInt("puntoventa_id");
						Integer retEmpresaID = rsetEventos.getInt("empresa_id");
						if(empresaID != null && !empresaID.equals("")){
							stmt.setInt("EMPRESA_ID", Integer.parseInt(empresaID));
						}
						Integer estadoId =  rsetEventos.getInt("estado_id");
						Date fecCorrida = rset.getDate("feccorrida");
						Boolean isOutros = rsetEventos.getString("desctipoevento") == "EXCESSO BAGAGEM" ? false : true;
						BigDecimal totalEvento = rsetEventos.getBigDecimal("total_evento");
					

						RelatorioComissaoSinteticoBean existingBean = findExistingBean(lsDadosRelatorio, agenciaID, retEmpresaID);
						if (existingBean != null) {
							if(isOutros) {
								existingBean.setTotalOutros(totalEvento);
							} else {
								existingBean.setTotalExcBagagem(totalEvento);
							}
							//aplica comissão dos totais
							if (configComissao != null) {
									comissaoCalc(configComissao, retEmpresaID, estadoId, fecCorrida, null, null, null, null, totalEvento, existingBean, isOutros);
							}
						}
					}
					caulcaComissaoTotal(lsDadosRelatorio);
					
					setLsDadosRelatorio(lsDadosRelatorio);
				}
			}

			private void comissaoCalc(LinkedHashMap<String, Object> configComissao, Integer retEmpresaID, Integer estadoId, Date fecCorrida, 
					BigDecimal baseCalculoTarifa, BigDecimal baseCalculoImpPost, BigDecimal baseCalculoVendaImpPost, BigDecimal baseCalculoSeguro, BigDecimal TotalEventos,
					RelatorioComissaoSinteticoBean comissaoBean, Boolean isEventoOutros) throws Exception {

				BigDecimal percComissaoTarifa = BigDecimal.ZERO;
				BigDecimal percComissaoSeguro = BigDecimal.ZERO;
				BigDecimal percComissaoImpPost = BigDecimal.ZERO;
				BigDecimal percComissaoVendaImpPost = BigDecimal.ZERO;
				BigDecimal percComissaoOutros = BigDecimal.ZERO;
				
				BigDecimal valorComissaoSeguro = BigDecimal.ZERO;
				BigDecimal valorComissaoTarifa = BigDecimal.ZERO;
				BigDecimal valorComissaoImpPost = BigDecimal.ZERO;
				BigDecimal valorComissaoVendaImpPost = BigDecimal.ZERO;
				BigDecimal valorComissaoExcBagagem = BigDecimal.ZERO;
				BigDecimal valorComissaoOutros = BigDecimal.ZERO;
				Boolean isAltaTemporada = false;
				
				if(fecCorrida != null) {
					String mes = new SimpleDateFormat("M").format(fecCorrida);
					HashMap<String, Object> configImposto = getConfigImposto((Integer) retEmpresaID, estadoId);
					isAltaTemporada = ((mes.equals("1") && (Boolean) configImposto.get("INDJANEIRO")) || (mes.equals("2") && (Boolean) configImposto.get("INDFEVEREIRO"))
							|| (mes.equals("3") && (Boolean) configImposto.get("INDMARCO")) || (mes.equals("4") && (Boolean) configImposto.get("INDABRIL"))
							|| (mes.equals("5") && (Boolean) configImposto.get("INDMAIO")) || (mes.equals("6") && (Boolean) configImposto.get("INDJUNHO"))
							|| (mes.equals("7") && (Boolean) configImposto.get("INDJULHO")) || (mes.equals("8") && (Boolean) configImposto.get("INDAGOSTO"))
							|| (mes.equals("9") && (Boolean) configImposto.get("INDSETEMBRO")) || (mes.equals("10") && (Boolean) configImposto.get("INDOUTUBRO"))
							|| (mes.equals("11") && (Boolean) configImposto.get("INDNOVEMBRO")) || (mes.equals("12") && (Boolean) configImposto.get("INDDEZEMBRO"))) ? true : false;
				}
				if (!isEventoOutros) {
					percComissaoTarifa = isAltaTemporada ? (BigDecimal) configComissao.get("PASSAGEMALTA") : (BigDecimal) configComissao.get("PASSAGEMBAIXA");
					percComissaoSeguro = isAltaTemporada ? (BigDecimal) configComissao.get("SEGUROALTA") : (BigDecimal) configComissao.get("SEGUROBAIXA");
					percComissaoImpPost = isAltaTemporada ? (BigDecimal) configComissao.get("VENDAIMPGAPALTA") : (BigDecimal) configComissao.get("VENDAIMPGAPBAIXA");
					percComissaoVendaImpPost = isAltaTemporada ? (BigDecimal) configComissao.get("VENDAGAPALTA") : (BigDecimal) configComissao.get("VENDAGAPBAIXA");
					
					if (percComissaoTarifa == null) {
						percComissaoTarifa = BigDecimal.ZERO;
					}
					if (percComissaoSeguro == null) {
						percComissaoSeguro = BigDecimal.ZERO;
					}

					if (percComissaoImpPost == null) {
						percComissaoImpPost = BigDecimal.ZERO;
					}
					if (percComissaoVendaImpPost == null) {
						percComissaoVendaImpPost = BigDecimal.ZERO;
					}

					// VALOR COMISSAO = BASE DE CALCULO * (PERCENTUAL COMISSAO / 100)
					valorComissaoTarifa = baseCalculoTarifa.multiply(MoneyHelper.dividir(percComissaoTarifa, MoneyHelper.HUNDRED, 4));
					valorComissaoSeguro = baseCalculoSeguro.multiply(MoneyHelper.dividir(percComissaoSeguro, MoneyHelper.HUNDRED, 4));
					valorComissaoImpPost = baseCalculoImpPost.multiply(MoneyHelper.dividir(percComissaoImpPost, MoneyHelper.HUNDRED, 4));
					valorComissaoVendaImpPost = baseCalculoVendaImpPost.multiply(MoneyHelper.dividir(percComissaoVendaImpPost, MoneyHelper.HUNDRED, 4));;
					
					if(comissaoBean.getComissaoTarifa() == null) {
						comissaoBean.setComissaoTarifa(valorComissaoTarifa);
					} else {
						comissaoBean.setComissaoTarifa(comissaoBean.getComissaoTarifa().add(valorComissaoTarifa));
					}
					if(comissaoBean.getComissaoSeguro() == null) {
						comissaoBean.setComissaoSeguro(valorComissaoSeguro);
					} else {
						comissaoBean.setComissaoSeguro(comissaoBean.getComissaoSeguro().add(valorComissaoSeguro));
					}
					if(comissaoBean.getComissaoImpPosterior() == null) {
						comissaoBean.setComissaoImpPosterior(valorComissaoImpPost);
					} else {
						comissaoBean.setComissaoImpPosterior(comissaoBean.getComissaoImpPosterior().add(valorComissaoImpPost));
					}
					if(comissaoBean.getComissaoVendaImpPosterior() == null) {
						comissaoBean.setComissaoVendaImpPosterior(valorComissaoVendaImpPost);
					} else {
						comissaoBean.setComissaoVendaImpPosterior(comissaoBean.getComissaoVendaImpPosterior().add(valorComissaoVendaImpPost));
					}
					
				} else {
					percComissaoOutros = isEventoOutros ? (isAltaTemporada ? (BigDecimal) configComissao.get("OUTROSALTA") : (BigDecimal) configComissao.get("OUTROSBAIXA") )
															:(isAltaTemporada ? (BigDecimal) configComissao.get("EXCESSOALTA") : (BigDecimal) configComissao.get("EXCESSOBAIXA"));
					if(percComissaoOutros == null) {
						percComissaoOutros = BigDecimal.ZERO;
					}
					if(isEventoOutros) {
						// VALOR COMISSAO = BASE DE CALCULO * (PERCENTUAL COMISSAO / 100)
						valorComissaoOutros = comissaoBean.getTotalOutros().multiply(MoneyHelper.dividir(percComissaoSeguro, MoneyHelper.HUNDRED, 4));
						if(comissaoBean.getComissaoOutros() == null) {
							comissaoBean.setComissaoOutros(valorComissaoOutros);
						} else {
							comissaoBean.setComissaoOutros(comissaoBean.getComissaoOutros().add(valorComissaoOutros));
						}
						comissaoBean.setComissaoOutros(comissaoBean.getComissaoOutros().add(valorComissaoOutros));
					} else {
						valorComissaoExcBagagem = comissaoBean.getTotalExcBagagem().multiply(MoneyHelper.dividir(percComissaoOutros, MoneyHelper.HUNDRED, 4));
						if(comissaoBean.getComissaoExcBagagem() == null) {
							comissaoBean.setComissaoExcBagagem(valorComissaoExcBagagem);
						} else {
							comissaoBean.setComissaoExcBagagem(comissaoBean.getComissaoExcBagagem().add(valorComissaoExcBagagem));

						}
					}
				}
			}
		});
	}
	
	public void setLsDadosRelatorio(List<RelatorioComissaoSinteticoBean> lsDadosRelatorio) {
		this.setCollectionDataSource(new JRBeanCollectionDataSource(lsDadosRelatorio));
		this.lsDadosRelatorio = lsDadosRelatorio;
	}

	private void caulcaComissaoTotal(List<RelatorioComissaoSinteticoBean> lsDadosRelatorio) {
		if(lsDadosRelatorio!=null && !lsDadosRelatorio.isEmpty()) {			
			for (RelatorioComissaoSinteticoBean bean : lsDadosRelatorio) {
				try {
					BigDecimal totalComissoes = bean.getComissaoTarifa()
							.add(bean.getComissaoImpPosterior()!=null ? bean.getComissaoImpPosterior() : BigDecimal.ZERO)
							.add(bean.getComissaoVendaImpPosterior()!=null ?bean.getComissaoVendaImpPosterior(): BigDecimal.ZERO)
							.add(bean.getComissaoExcBagagem()!=null ? bean.getComissaoExcBagagem(): BigDecimal.ZERO)
							.add(bean.getComissaoOutros()!=null ? bean.getComissaoOutros(): BigDecimal.ZERO);
					bean.setComissaoTotal(totalComissoes);
				}catch(ArrayIndexOutOfBoundsException | NullPointerException e) {
					log.error("Erro a gerar relatorio Comissao sintetico", e);
				}
			}
		}
	}
	
	public HashMap<String, Object> getConfigImposto(Integer empresaId, Integer estadoId) throws Exception {
		HashMap<String, Object> cacheConfig = null;

		try {
			cacheConfig = (HashMap<String, Object>) CalculoImposto.getConfigImpostoByEstadoOrigem(this.getConexao(), estadoId, empresaId);
		} catch (Exception e) {
			if (e instanceof ExceptionConfiguracao)
				this.addInfoMsg(e.getMessage());
			else
				throw e;
		}

		return cacheConfig;
	}

    private static RelatorioComissaoSinteticoBean findExistingBean(List<RelatorioComissaoSinteticoBean> list, Integer agenciaID, Integer empresaID) {
        for (RelatorioComissaoSinteticoBean bean : list) {
            if (Objects.equals(bean.getPontoVendaID(), agenciaID) && Objects.equals(bean.getEmpresaID(), empresaID)) {
                return bean;
            }
        }
        return null;
    }
	
	@Override
	protected void processaParametros() throws Exception {
	}
	
	private String getSqlDadosTotaisdeVenda(String empresa, String agencia) {
	
		StringBuilder sql = new StringBuilder(3400);
		
		sql.append(" SELECT pv.nombpuntoventa,                                                                                                	");
		sql.append("	b.puntoventa_id,																								  		");
		sql.append("	ep.empresa_id,																									  		");
		sql.append("	ci.estado_id,																								      		");
		sql.append("	b.feccorrida,																									  		");
		sql.append("  	Count(*),                                                                                                         		");
		sql.append("   	SUM(CASE WHEN (b.indreimpresion = 1 AND b.tipoventa_id = 18 or (b.indreimpresion = 0 AND b.tipoventa_id = 18)) 	  		"); 
		sql.append("	THEN 0 ELSE b.preciobase END) AS total_passagem,															  			");
		sql.append("   	SUM(CASE WHEN b.indreimpresion = 1 AND b.tipoventa_id = 18 THEN b.preciobase ELSE 0 END) AS total_imp_posterior,  		");
		sql.append("   	SUM(CASE WHEN b.indreimpresion = 0 AND b.tipoventa_id = 18 THEN b.preciobase ELSE 0 END) AS total_venda_imp_posterior, 	");
		sql.append("    SUM(b.importeseguro)            AS total_seguro,                                                                  		");
		sql.append("    SUM(b.importetaxaembarque)      AS total_tax_emb,                                                                 		");
		sql.append("    SUM(b.importepedagio)           AS total_pedagio,                                                                 		");
		sql.append("    SUM(b.importeoutros)            AS total_outros,                                                                  		");
		sql.append("    SUM(b.preciopagado)             AS total_venda                                                                    		");
		sql.append(" FROM   boleto b                                                                                                          	");
		sql.append("        LEFT JOIN punto_venta       pv  ON (b.puntoventa_id = pv.puntoventa_id)                                           	");
		sql.append("		LEFT JOIN marca             mc  ON (b.marca_id = mc.marca_id)													  	");
		sql.append("        LEFT JOIN empresa           ep  ON (mc.empresa_id = ep.empresa_id)												  	");
		sql.append("        LEFT JOIN ciudad            ci  ON (ep.ciudad_id = ci.ciudad_id)												  	");
		sql.append(" WHERE  (b.motivocancelacion_id IS NULL or (b.motivocancelacion_id = 16 and b.tipoventa_id =18 ))	                      	");
		sql.append(" 	    AND b.motivoreimpresion_id IS NULL                                                                                	");
		sql.append("        AND (b.boletooriginal_id IS NULL or ( b.indreimpresion = 1 AND b.tipoventa_id =18) )                              	");
		sql.append("        AND b.activo = 1                                                                                                  	");
		sql.append("        AND b.fechorventa BETWEEN :DATA_INICIAL AND :DATA_FINAL  														  	");
		if (!agencia.isEmpty()) {
			sql.append("    AND b.puntoventa_id in ("+agencia+")	                                                                            ");
		}
		if (!empresa.isEmpty()) {
			sql.append("    AND ep.empresa_id = :EMPRESA_ID         								                                          	");
		}
		sql.append(" GROUP  BY pv.nombpuntoventa,  b.puntoventa_id, ep.empresa_id, ci.estado_id, b.feccorrida                                 	");
		sql.append(" ORDER  BY pv.nombpuntoventa,  b.puntoventa_id, ci.estado_id      	                                                      	");
		
		return sql.toString();
	}
	
	private String getSqlDadosTotaisEvento(String empresa, String agencia ) {
		
		StringBuilder sql = new StringBuilder(3400);
		
		sql.append("SELECT                                                                                      ");
		sql.append("    tee.desctipoevento,                                                                     ");
		sql.append("    ee.puntoventa_id,                                                                       ");
		sql.append("    pv.nombpuntoventa,                                                                      ");
		sql.append("	ee.empresa_id,																			");
		sql.append("	ci.estado_id,																			");
		sql.append("	bo.fechorventa,																			");
		sql.append("    SUM(ee.impingreso) AS total_evento                                                      ");
		sql.append("FROM                                                                                        ");
		sql.append("         evento_extra ee                                                                    ");
		sql.append("    INNER JOIN tipo_evento_extra  tee ON ee.tipoeventoextra_id = tee.tipoeventoextra_id     ");
		sql.append("    LEFT JOIN punto_venta        pv ON ( ee.puntoventa_id = pv.puntoventa_id )              ");
		sql.append(" 	LEFT JOIN empresa emp ON (emp.empresa_id = ee.empresa_id) 								");
		sql.append("    LEFT JOIN ciudad ci ON (emp.ciudad_id = ci.ciudad_id)									");
		sql.append("    LEFT JOIN boleto bo ON (ee.boleto_id = bo.boleto_id)									");
		sql.append("WHERE                                                                                       ");
		sql.append("        ee.activo = 1                                                                       ");
		sql.append("    AND tee.desctipoevento IN ('EXCESSO BAGAGEM', 'ANIMAL 50%' )                            ");
		sql.append("    AND ee.fechoringreso BETWEEN :DATA_INICIAL AND :DATA_FINAL								");

		if (!agencia.isEmpty()) {
			sql.append("    AND ee.puntoventa_id IN ("+agencia+")                                               ");
		}
		if (!empresa.isEmpty()) {
			sql.append("    AND ee.empresa_id = :EMPRESA_ID                                                     ");
		}
 		
		sql.append("GROUP BY                                                                                    ");
		sql.append("    tee.desctipoevento,                                                                     ");
		sql.append("    ee.puntoventa_id,                                                                       ");
		sql.append("    pv.nombpuntoventa,                                                                      ");
		sql.append("	ee.empresa_id,																			");
		sql.append("	ci.estado_id,																			");	
		sql.append("	bo.fechorventa																			");
		sql.append("ORDER BY                                                                                    ");
		sql.append("    tee.desctipoevento,                                                                     ");
		sql.append("    ee.puntoventa_id,                                                                       ");
		sql.append("    pv.nombpuntoventa,                                                                      ");
		sql.append("	ee.empresa_id																			");	
		
		return sql.toString();
	}
	
	
	public LinkedHashMap<String, Object> getConfigComissao(Integer puntoVentaId, Integer empresaId) throws Exception {

		LinkedHashMap<String, Object> cacheConfig = null;

		// Verifica se existe configuração na memoria, caso não exista, realiza busca no banco
		if (mapCacheConfigComissao == null || !mapCacheConfigComissao.containsKey(puntoVentaId.toString() + "_" + empresaId.toString())) {
			StringBuilder sql = new StringBuilder();

			sql.append(" SELECT PC.ISSRETIDO, PC.ROYALTIES, PC.ENVIARRECIBO, PC.RECEITA, PC.CODAG, PC.PASSAGEMALTA, PC.PASSAGEMBAIXA, 	 									");
			sql.append("        PC.SEGUROALTA, PC.SEGUROBAIXA, PC.OUTROSBAIXA, PC.OUTROSALTA, PC.EXCESSOALTA, PC.EXCESSOBAIXA, 												");
			sql.append("        PC.TARIFARECEITA, PC.SEGURORECEITA, PC.TAXARECEITA, PC.PEDAGIORECEITA, TP.DESCTIPO AS TIPO_AGENCIA,  										");
			sql.append("        PC.TARIFADEV, PC.SEGURO_DEV, PC.TAXADEV, PC.PEDAGIODEV, PV.PUNTOVENTA_ID, PV.NUMPUNTOVENTA, PV.NOMBPUNTOVENTA,  							");
			sql.append("        ES.CVEESTADO, ES.ESTADO_ID, PC.SEGUROOBRIGATORIOALTA, PC.SEGUROOBRIGATORIOBAIXA,														 	");
			sql.append("        CASE WHEN pc.valor_imp_gap_alta_comp IS NULL THEN pc.valor_impressao_gap_alta ELSE pc.valor_imp_gap_alta_comp END AS VALOR_IMPRESSAO_ALTA, 	");
			sql.append("        CASE WHEN pc.valor_imp_gap_comp IS NULL THEN pc.valor_impressao_gap ELSE pc.valor_imp_gap_comp END AS VALOR_IMPRESSAO_BAIXA, 				");
			sql.append("        CASE WHEN pc.valor_venda_gap_alta_comp IS NULL THEN pc.passagemalta ELSE pc.valor_venda_gap_alta_comp END AS VALOR_VENDA_IMP_ALTA, 			");
			sql.append("        CASE WHEN pc.valor_venda_gap_comp IS NULL THEN pc.passagembaixa ELSE pc.valor_venda_gap_comp END AS VALOR_VENDA_IMP_BAIXA 					");
			sql.append("   FROM PTOVTA_COMISSAO PC ");
			sql.append("   INNER JOIN PUNTO_VENTA     PV ON PV.PUNTOVENTA_ID = PC.PUNTOVENTA_ID AND PC.PTOVTADESCOMISSAO_ID = PV.PUNTOVENTA_ID ");
			sql.append("   INNER JOIN PARADA          PR ON PR.PARADA_ID = PV.PARADA_ID ");
			sql.append("   INNER JOIN CIUDAD          CD ON CD.CIUDAD_ID = PR.CIUDAD_ID ");
			sql.append("   INNER JOIN ESTADO          ES ON ES.ESTADO_ID = CD.ESTADO_ID ");
			sql.append("   INNER JOIN TIPO_PTOVTA     TP ON TP.TIPOPTOVTA_ID = PV.TIPOPTOVTA_ID");
			sql.append("  WHERE PC.EMPRESA_ID = :EMPRESA_ID ");
			sql.append("    AND PC.PUNTOVENTA_ID = :PUNTOVENTA_ID ");
			sql.append("    AND PC.ACTIVO = 1 ");

			NamedParameterStatement stmt = new NamedParameterStatement(this.getConexao(), sql.toString());
			stmt.setInt("EMPRESA_ID", empresaId);
			stmt.setInt("PUNTOVENTA_ID", puntoVentaId);
			ResultSet rs = stmt.executeQuery();

			if (rs.next()) {
				cacheConfig = new LinkedHashMap<String, Object>();

				cacheConfig.put("ISSRETIDO", rs.getBigDecimal("ISSRETIDO"));
				cacheConfig.put("ROYALTIES", rs.getBigDecimal("ROYALTIES"));
				cacheConfig.put("PASSAGEMALTA", rs.getBigDecimal("PASSAGEMALTA"));
				cacheConfig.put("PASSAGEMBAIXA", rs.getBigDecimal("PASSAGEMBAIXA"));
				cacheConfig.put("SEGUROALTA", rs.getBigDecimal("SEGUROALTA"));
				cacheConfig.put("SEGUROBAIXA", rs.getBigDecimal("SEGUROBAIXA"));
				cacheConfig.put("OUTROSALTA", rs.getBigDecimal("OUTROSALTA"));
				cacheConfig.put("OUTROSBAIXA", rs.getBigDecimal("OUTROSBAIXA"));
				cacheConfig.put("EXCESSOALTA", rs.getBigDecimal("EXCESSOALTA"));
				cacheConfig.put("EXCESSOBAIXA", rs.getBigDecimal("EXCESSOBAIXA"));
				cacheConfig.put("SEGUROOBRIGATORIOBAIXA", rs.getBigDecimal("SEGUROOBRIGATORIOBAIXA"));
				cacheConfig.put("SEGUROOBRIGATORIOALTA", rs.getBigDecimal("SEGUROOBRIGATORIOALTA"));
				cacheConfig.put("VENDAIMPGAPALTA", rs.getBigDecimal("VALOR_IMPRESSAO_ALTA"));
				cacheConfig.put("VENDAIMPGAPBAIXA", rs.getBigDecimal("VALOR_IMPRESSAO_BAIXA"));
				cacheConfig.put("VENDAGAPALTA", rs.getBigDecimal("VALOR_VENDA_IMP_ALTA"));
				cacheConfig.put("VENDAGAPBAIXA", rs.getBigDecimal("VALOR_VENDA_IMP_BAIXA"));
				
				cacheConfig.put("TARIFARECEITA", rs.getBoolean("TARIFARECEITA"));
				cacheConfig.put("SEGURORECEITA", rs.getBoolean("SEGURORECEITA"));
				cacheConfig.put("TAXARECEITA", rs.getBoolean("TAXARECEITA"));
				cacheConfig.put("PEDAGIORECEITA", rs.getBoolean("PEDAGIORECEITA"));
				cacheConfig.put("TARIFADEV", rs.getBoolean("TARIFADEV"));
				cacheConfig.put("SEGURO_DEV", rs.getBoolean("SEGURO_DEV"));
				cacheConfig.put("TAXADEV", rs.getBoolean("TAXADEV"));
				cacheConfig.put("PEDAGIODEV", rs.getBoolean("PEDAGIODEV"));
				
				

				cacheConfig.put("RECEITA", rs.getString("RECEITA"));
				cacheConfig.put("TIPO_AGENCIA", rs.getString("TIPO_AGENCIA"));
				cacheConfig.put("NUMPUNTOVENTA", rs.getString("NUMPUNTOVENTA"));
				cacheConfig.put("NOMBPUNTOVENTA", rs.getString("NOMBPUNTOVENTA"));
				cacheConfig.put("CVEESTADO", rs.getString("CVEESTADO"));

				cacheConfig.put("ESTADO_ID", rs.getInt("ESTADO_ID"));
				cacheConfig.put("PUNTOVENTA_ID", rs.getInt("PUNTOVENTA_ID"));

				mapCacheConfigComissao.put(puntoVentaId.toString() + "_" + empresaId.toString(), cacheConfig);

			} else {
				StringBuilder sqlParam = new StringBuilder();

				sqlParam.append(" SELECT EM.NOMBEMPRESA, PV.NOMBPUNTOVENTA, PV.PUNTOVENTA_ID, PV.NUMPUNTOVENTA ");
				sqlParam.append("   FROM EMPRESA 	 EM, ");
				sqlParam.append("        PUNTO_VENTA PV ");
				sqlParam.append("  WHERE EM.EMPRESA_ID = :EMPRESA_ID ");
				sqlParam.append("    AND PV.PUNTOVENTA_ID = :PUNTOVENTA_ID ");

				NamedParameterStatement stmtParam = new NamedParameterStatement(this.getConexao(), sqlParam.toString());
				stmtParam.setInt("PUNTOVENTA_ID", puntoVentaId);
				stmtParam.setInt("EMPRESA_ID", empresaId);

				ResultSet rsParam = stmtParam.executeQuery();

				if (rsParam.next()){
					pontoVendaConfiguracao.add(new PuntoVenta(new Integer (rsParam.getString("PUNTOVENTA_ID")), rsParam.getString("NOMBPUNTOVENTA"), rsParam.getString("NUMPUNTOVENTA")));
				}
				rsParam.close();
				stmtParam.close();
			}

			rs.close();
			stmt.close();

		}
		else
			cacheConfig = mapCacheConfigComissao.get(puntoVentaId.toString() + "_" + empresaId.toString());

		return cacheConfig;

	}
	
}