package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.ItemAdicional;
import com.rjconsultores.ventaboletos.service.ItemAdicionalService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderItemAdicional;

@Controller("busquedaConfiguracionItensPacoteController")
@Scope("prototype")
public class BusquedaConfiguracionItensPacoteController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private transient PagedListWrapper<ItemAdicional> plwItemPacote;

	@Autowired
	private ItemAdicionalService itemAdicionalService;

	private MyTextbox txtDescItemAdicional;
	private MyListbox itemAdicionalList;
	private Paging pagingItemAdicional;

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		itemAdicionalList.setItemRenderer(new RenderItemAdicional());
		itemAdicionalList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				ItemAdicional item = (ItemAdicional) itemAdicionalList.getSelected();
				selecionarItemAdicional(item);
			}
		});

		refreshListaItemAdicional();

	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void selecionarItemAdicional(ItemAdicional item) {
		if (item == null) {
			return;
		}

		Map args = new HashMap();
		args.put("itemAdicional", item);
		args.put("itemAdicionalList", itemAdicionalList);

		openWindow("/gui/configuraciones_comerciales/editarItemAdicional.zul",
				Labels.getLabel("editarConfiguracionPacoteController.window.title"), args, MODAL);
	}

	private void refreshListaItemAdicional() {
		HibernateSearchObject<ItemAdicional> itemBusqueda =
				new HibernateSearchObject<ItemAdicional>(ItemAdicional.class, pagingItemAdicional.getPageSize());

		itemBusqueda.addFilterLike("descitemadicional", "%" + txtDescItemAdicional.getText().trim().concat("%"));
		itemBusqueda.addFilterEqual("activo", Boolean.TRUE);
		itemBusqueda.addSortAsc("descitemadicional");

		plwItemPacote.init(itemBusqueda, itemAdicionalList, pagingItemAdicional);
	}

	public void onClick$btnRefresh(Event ev) throws InterruptedException {
		refreshListaItemAdicional();
	}

	public void onClick$btnAddItem(Event ev) throws InterruptedException {
		selecionarItemAdicional(new ItemAdicional());
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshListaItemAdicional();
	}

	public void onClick$btnRemoverItem(Event ev) throws InterruptedException {

		int resp = Messagebox.show(
				Labels.getLabel("editarConfiguracionPacoteController.itemAdicional.MSG.borrarPergunta"),
				Labels.getLabel("editarConfiguracionPacoteController.window.title"),
				Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

		if (resp == Messagebox.YES) {

			ItemAdicional item = (ItemAdicional) itemAdicionalList.getSelected();
			itemAdicionalService.borrar(item);

			Messagebox.show(
					Labels.getLabel("editarConfiguracionPacoteController.itemAdicional.MSG.borrarOK"),
					Labels.getLabel("editarConfiguracionPacoteController.window.title"),
					Messagebox.OK, Messagebox.INFORMATION);

			itemAdicionalList.removeItem(item);
		}
	}
}
